<?php

/**
 * Szöveg tördelése és kirajzolása - rekurzív (sorok száma-1 leszállás)
 * @param image resource $im - A kép, amire rajzoljuk a szöveget
 * @param int $size - A kiírandó szöveg mérete pontban
 * @param int $angle - A szöveg dőlésszöge fokban
 * @param int $x_pos - A szöveg pozíciójának X koordinátája
 * @param int $y_pos - A szöveg pozíciójának Y koordinátája
 * @param int $color - A szöveg színe
 * @param string $font - A szöveg betűtípusának URL-je (.ttf fájl)
 * @param string $instr - A kiírandó szöveg
 */
function imagettftextwrap($im, $size, $angle, $x_pos, $y_pos, $color, $font, $instr) {
    $box = @imagettfbbox($size, 0, $font, $instr);
    $width = abs($box[4] - $box[0]);
    $height = abs($box[3] - $box[5]);
    $overlap = (($x_pos + $width) - imagesx($im));
    if ($overlap > 0) { //ha a szöveg nem fér ki: tördelés
        $words = explode(' ', $instr);
        $str = "";
        $str_buffer = "";
        for ($m = 0; $m < sizeof($words); $m++) {
            $bo = imagettfbbox($size, 0, $font, $str);
            $wid = abs($bo[4] - $bo[0]);
            if (($x_pos + $wid) < imagesx($im)) { // addig adunk hozzá új karaktert, amíg nem ér túl
                $str_buffer .= $words[$m];
                $bo2 = imagettfbbox($size, 0, $font, $str_buffer);
                $wid2 = abs($bo2[4] - $bo2[0]);
                if (($x_pos + $wid2) < imagesx($im)) {
                    $str .= $words[$m];
                } else if ($m == 0) { // egy szó kilóg: karakternél törünk
                    $str_buffer = $str;   // induljunk a hosszú szó előtti stringből
                    $chars = str_split($words[$m]);
                    for ($i = 0; $i < count($chars); $i++) {
                        $str_buffer .= $chars[$i];
                        $bo3 = imagettfbbox($size, 0, $font, $str_buffer);
                        $wid3 = abs($bo3[4] - $bo3[0]);
                        if (($x_pos + $wid3) < imagesx($im)) { // még nem kell törni
                            $str .= $chars[$i];
                        } else { //törés
                            $word_chunk = array_slice($chars, $i); // a szó maradékát meghatározzuk..
                            if (empty($word_chunk)) {
                                array_shift($words);    // ..ha elfogyott a szó, kivesszük
                            } else {
                                $words[$m] = implode('', $word_chunk);  // ..különben visszaírjuk a maradékot
                            }
                            break;
                        }
                    }
                    break;
                } else {
                    break;
                }
            } else {
                break;
            }
            $str .= " "; // ide csak akkor érünk, ha nincs sortörés és egy szót feldolgoztunk -> space
        }
        $restof = "";
        for ($l = $m; $l < sizeof($words); $l++) {
            $restof .= $words[$l] . ' '; // a szöveg maradékát hozzáadjuk az új sorhoz
        }

        imagettftext($im, $size, $angle, $x_pos, $y_pos, $color, $font, $str); // aktuális sor kiírása

        if (!empty($restof)) {
            imagettftextwrap($im, $size, $angle, $x_pos, $y_pos + $height, $color, $font, $restof); // REKURZIÓ: a maradék szöveggel
        }
    } else {
        imagettftext($im, $size, $angle, $x_pos, $y_pos, $color, $font, $instr); // ha egysoros, hívjuk az alap függvényt
    }
}

// függvény vége: imagettftextwrap()


if (!isset($_GET['text'])) {
    error_log('ttftext.php: nincs megadva szöveg');
    exit(1);
}

// A program kimenete maga a PNG kép lesz
header('Content-Type: image/png');

$text = urldecode($_GET['text']);
$font = './stonehenge.ttf';
$size = (isset($_GET['size']) && is_numeric($_GET['size'])) ? $_GET['size'] : 18;

// Kiszámoljuk a kép méreteit.
$bounds = imagettfbbox($size, 0, $font, $text);

// Maximális méret megadása, hogy ne lóghasson túl az adott honlapon
$maxwidth = 580;

// Elkészítjük a képet. A méret attól függ, hogy hány sorba kell tördelni.
if ($bounds[2] + 5 > $maxwidth) {
    $im = imagecreatetruecolor($maxwidth, (floor($bounds[2] / $maxwidth) + 1) * $size * 1.8);
} else {
    $im = imagecreatetruecolor($bounds[2] + 5, $size * 1.8);
}


// Szín elkészítése: sötétvörös
$color = imagecolorallocate($im, 0x32, 0x00, 0x00);

// A háttérszín átlátszó: ezzel kitöltünk
$bgcolor = imagecolorallocatealpha($im, 255, 0, 0, 127);
imagesavealpha($im, true);
imagefill($im, 0, 0, $bgcolor);

// A tördelt szöveg hozzáírása
imagettftextwrap($im, $size, 0, 1, $size * 1.3, $color, $font, stripslashes($text));

// Kép kirajzolása kimenetre
imagepng($im);

// Takarítás
imagedestroy($im);
?>